@@ -20,6 +20,9 @@
  *
  */
 
+/* DPMS support has been moved to gnome-power-manager */
+#undef USE_LEGACY_DPMS_SUPPORT
+
 #include "config.h"
 #include <stdlib.h>
 #include <stdio.h>
@@ -34,7 +37,11 @@
 #include "gs-manager.h"
 #include "gs-watcher.h"
 #include "gs-fade.h"
+
+#ifdef USE_LEGACY_DPMS_SUPPORT
 #include "gs-power.h"
+#endif
+
 #include "gs-listener-dbus.h"
 #include "gs-monitor.h"
 #include "gs-prefs.h"
@@ -51,7 +58,9 @@ struct GSMonitorPrivate
         GSWatcher      *watcher;
         GSListener     *listener;
         GSManager      *manager;
+#ifdef USE_LEGACY_DPMS_SUPPORT
         GSPower        *power;
+#endif
         GSPrefs        *prefs;
         GSFade         *fade;
 };
@@ -86,49 +95,64 @@ manager_activated_cb (GSManager *manager,
 
 static void
 manager_deactivated_cb (GSManager *manager,
-                         GSMonitor *monitor)
+                        GSMonitor *monitor)
 {
         gs_listener_set_active (monitor->priv->listener, FALSE);
 }
 
 static gboolean
 watcher_idle_cb (GSWatcher *watcher,
-                 int        reserved,
+                 gboolean   is_idle,
                  GSMonitor *monitor)
 {
         gboolean res;
-        gboolean fade_active;
 
-        fade_active = gs_fade_get_active (monitor->priv->fade);
+        gs_debug ("Idle signal detected: %d", is_idle);
 
-        res = gs_listener_set_idle (monitor->priv->listener, TRUE);
-
-        if (fade_active) {
-                gs_fade_set_active (monitor->priv->fade, FALSE);
-        }
+        res = gs_listener_set_session_idle (monitor->priv->listener, is_idle);
 
         return res;
 }
 
 static gboolean
 watcher_idle_notice_cb (GSWatcher *watcher,
-                        int        reserved,
+                        gboolean   in_effect,
                         GSMonitor *monitor)
 {
-        /* start slow fade */
-        gs_fade_set_timeout (monitor->priv->fade, FADE_TIMEOUT);
-        gs_fade_set_active (monitor->priv->fade, TRUE);
+        gboolean activation_enabled;
+        gboolean inhibited;
+        gboolean handled;
 
-        return TRUE;
-}
+        gs_debug ("Idle notice signal detected: %d", in_effect);
 
-static void
-watcher_notice_cancelled_cb (GSWatcher *watcher,
-                             GSMonitor *monitor)
-{
-        /* cancel the fade */
-        gs_fade_set_active (monitor->priv->fade, FALSE);
-        gs_fade_reset (monitor->priv->fade);
+        /* only fade if screensaver can activate */
+        activation_enabled = gs_listener_get_activation_enabled (monitor->priv->listener);
+        inhibited = gs_listener_is_inhibited (monitor->priv->listener);
+
+        handled = FALSE;
+        if (in_effect) {
+                if (activation_enabled && ! inhibited) {
+                        /* start slow fade */
+                        gs_fade_set_timeout (monitor->priv->fade, FADE_TIMEOUT);
+                        gs_fade_set_active (monitor->priv->fade, TRUE);
+                        handled = TRUE;
+                }
+        } else {
+                gboolean manager_active;
+
+                manager_active = gs_manager_get_active (monitor->priv->manager);
+                /* cancel the fade unless manager was activated */
+                if (! manager_active) {
+                        gs_fade_set_active (monitor->priv->fade, FALSE);
+                        gs_fade_reset (monitor->priv->fade);
+                } else {
+                        gs_debug ("manager active, skipping fade cancellation");
+                }
+
+                handled = TRUE;
+        }
+
+        return handled;
 }
 
 static void
@@ -141,7 +165,7 @@ listener_lock_cb (GSListener *listener,
         if (res) {
                 gs_manager_set_lock_active (monitor->priv->manager, TRUE);
         } else {
-                g_warning ("Unable to lock the screen");
+                gs_debug ("Unable to lock the screen");
         }
 }
 
@@ -166,67 +190,40 @@ listener_active_changed_cb (GSListener *listener,
                             GSMonitor  *monitor)
 {
         gboolean res;
+        gboolean ret;
         gboolean idle_watch_enabled;
 
-        idle_watch_enabled = gs_watcher_get_enabled (monitor->priv->watcher);
-
-        if (active) {
-                if (idle_watch_enabled) {
-                        /* turn off the idleness watcher */
-                        res = gs_watcher_set_active (monitor->priv->watcher, FALSE);
-                        if (! res) {
-                                g_warning ("Unable to deactivate the idle watcher");
-                                return FALSE;
-                        }
-                }
-
-                /* blank the screen */
-                res = gs_manager_set_active (monitor->priv->manager, TRUE);
-                if (! res) {
-                        g_warning ("Unable to blank the screen");
-
-                        /* since we can't activate then reactivate the watcher
-                           and give up */
-                        if (idle_watch_enabled) {
-                                res = gs_watcher_set_active (monitor->priv->watcher, TRUE);
-                        }
-
-                        return FALSE;
-                }
+        res = gs_manager_set_active (monitor->priv->manager, active);
+        if (! res) {
+                gs_debug ("Unable to set manager active: %d", active);
+                ret = FALSE;
+                goto done;
+        }
 
+#ifdef USE_LEGACY_DPMS_SUPPORT
                 /* enable power management */
-                res = gs_power_set_active (monitor->priv->power, TRUE);
+                res = gs_power_set_active (monitor->priv->power, active);
                 if (! res) {
-                        g_warning ("Unable to activate power management");
+                        gs_debug ("Unable to set power management active: %d", active);
 
                         /* if we can't activate power management it isn't the
                            end of the world */
                 }
-        } else {
-                /* unblank the screen */
-                res = gs_manager_set_active (monitor->priv->manager, FALSE);
-                if (! res) {
-                        g_warning ("Unable to unblank the screen");
-                        return FALSE;
-                }
+#endif
 
-                if (idle_watch_enabled) {
-                        /* turn on the idleness watcher */
-                        res = gs_watcher_set_active (monitor->priv->watcher, TRUE);
-                        if (! res) {
-                                g_warning ("Unable to activate the idle watcher");
-                                return FALSE;
-                        }
-                }
+        ret = TRUE;
 
-                /* disable power management */
-                res = gs_power_set_active (monitor->priv->power, FALSE);
+ done:
+
+        idle_watch_enabled = gs_watcher_get_enabled (monitor->priv->watcher);
+        if (ret && idle_watch_enabled) {
+                res = gs_watcher_set_active (monitor->priv->watcher, !active);
                 if (! res) {
-                        g_warning ("Unable to deactivate power management");
+                        gs_debug ("Unable to set the idle watcher active: %d", !active);
                 }
         }
 
-        return TRUE;
+        return ret;
 }
 
 static void
@@ -245,15 +242,18 @@ listener_poke_cb (GSListener *listener,
            idle watcher */
         gs_watcher_reset (monitor->priv->watcher);
 
+#ifdef USE_LEGACY_DPMS_SUPPORT
         /* turn on the monitor power */
         gs_power_set_mode (monitor->priv->power,
                            GS_POWER_MODE_ON);
+#endif
 
         /* request that the manager unlock -
            will pop up a dialog if necessary */
         gs_manager_request_unlock (monitor->priv->manager);
 }
 
+#ifdef USE_LEGACY_DPMS_SUPPORT
 static void
 power_changed_cb (GSPower    *power,
                   GSPowerMode mode,
@@ -272,6 +272,7 @@ power_changed_cb (GSPower    *power,
                 gs_manager_set_throttle_enabled (monitor->priv->manager, TRUE);
         }
 }
+#endif
 
 static void
 _gs_monitor_update_from_prefs (GSMonitor *monitor,
@@ -292,14 +293,20 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
         gs_manager_set_mode (monitor->priv->manager, monitor->priv->prefs->mode);
         gs_manager_set_themes (monitor->priv->manager, monitor->priv->prefs->themes);
 
-        idle_detection_enabled = (monitor->priv->prefs->mode != GS_MODE_DONT_BLANK);
+        /* enable activation when allowed */
+        gs_listener_set_activation_enabled (monitor->priv->listener,
+                                            monitor->priv->prefs->idle_activation_enabled);
+
+        /* idle detection always enabled */
+        idle_detection_enabled = TRUE;
+
         gs_watcher_set_timeout (monitor->priv->watcher, monitor->priv->prefs->timeout);
         gs_watcher_set_enabled (monitor->priv->watcher, idle_detection_enabled);
 
         /* in the case where idle detection is reenabled we may need to
            activate the watcher too */
 
-        manager_active = gs_manager_is_active (monitor->priv->manager);
+        manager_active = gs_manager_get_active (monitor->priv->manager);
         idle_detection_active = gs_watcher_get_active (monitor->priv->watcher);
         activate_watch = (! manager_active
                           && ! idle_detection_active
@@ -308,12 +315,15 @@ _gs_monitor_update_from_prefs (GSMonitor *monitor,
                 gs_watcher_set_active (monitor->priv->watcher, TRUE);
         }
 
+
+#ifdef USE_LEGACY_DPMS_SUPPORT
         gs_power_set_timeouts (monitor->priv->power,
                                monitor->priv->prefs->dpms_standby,
                                monitor->priv->prefs->dpms_suspend,
                                monitor->priv->prefs->dpms_off);
         gs_power_set_enabled (monitor->priv->power,
                               monitor->priv->prefs->dpms_enabled);
+#endif
 }
 
 static void
@@ -349,18 +359,15 @@ disconnect_watcher_signals (GSMonitor *monitor)
 {
         g_signal_handlers_disconnect_by_func (monitor->priv->watcher, watcher_idle_cb, monitor);
         g_signal_handlers_disconnect_by_func (monitor->priv->watcher, watcher_idle_notice_cb, monitor);
-        g_signal_handlers_disconnect_by_func (monitor->priv->watcher, watcher_notice_cancelled_cb, monitor);
 }
 
 static void
 connect_watcher_signals (GSMonitor *monitor)
 {
-        g_signal_connect (monitor->priv->watcher, "idle",
+        g_signal_connect (monitor->priv->watcher, "idle_changed",
                           G_CALLBACK (watcher_idle_cb), monitor);
-        g_signal_connect (monitor->priv->watcher, "idle_notice",
+        g_signal_connect (monitor->priv->watcher, "idle_notice_changed",
                           G_CALLBACK (watcher_idle_notice_cb), monitor);
-        g_signal_connect (monitor->priv->watcher, "notice_cancelled",
-                          G_CALLBACK (watcher_notice_cancelled_cb), monitor);
 }
 
 static void
@@ -379,6 +386,7 @@ connect_manager_signals (GSMonitor *monitor)
                           G_CALLBACK (manager_deactivated_cb), monitor);
 }
 
+#ifdef USE_LEGACY_DPMS_SUPPORT
 static void
 disconnect_power_signals (GSMonitor *monitor)
 {
@@ -391,6 +399,7 @@ connect_power_signals (GSMonitor *monitor)
         g_signal_connect (monitor->priv->power, "changed",
                           G_CALLBACK (power_changed_cb), monitor);
 }
+#endif
 
 static void
 disconnect_prefs_signals (GSMonitor *monitor)
@@ -424,8 +433,10 @@ gs_monitor_init (GSMonitor *monitor)
         monitor->priv->manager = gs_manager_new ();
         connect_manager_signals (monitor);
 
+#ifdef USE_LEGACY_DPMS_SUPPORT
         monitor->priv->power = gs_power_new ();
         connect_power_signals (monitor);
+#endif
 
         _gs_monitor_update_from_prefs (monitor, monitor->priv->prefs);
 }
@@ -445,14 +456,18 @@ gs_monitor_finalize (GObject *object)
         disconnect_watcher_signals (monitor);
         disconnect_listener_signals (monitor);
         disconnect_manager_signals (monitor);
+#ifdef USE_LEGACY_DPMS_SUPPORT
         disconnect_power_signals (monitor);
+#endif
         disconnect_prefs_signals (monitor);
 
         g_object_unref (monitor->priv->fade);
         g_object_unref (monitor->priv->watcher);
         g_object_unref (monitor->priv->listener);
         g_object_unref (monitor->priv->manager);
+#ifdef USE_LEGACY_DPMS_SUPPORT
         g_object_unref (monitor->priv->power);
+#endif
         g_object_unref (monitor->priv->prefs);
 
         G_OBJECT_CLASS (parent_class)->finalize (object);
@@ -478,7 +493,5 @@ gs_monitor_start (GSMonitor *monitor,
                 return FALSE;
         }
 
-        gs_listener_set_active (monitor->priv->listener, FALSE);
-
         return TRUE;
 }
